# ProcessNotify

Notifications for Process connections.


```pony
interface ref ProcessNotify
```

## Public Functions

### created

ProcessMonitor calls this when it is created.


```pony
fun ref created(
  process: ProcessMonitor ref)
: None val
```
#### Parameters

*   process: [ProcessMonitor](process-ProcessMonitor) ref

#### Returns

* [None](builtin-None) val

---

### stdout

ProcessMonitor calls this when new data is received on STDOUT of the
forked process


```pony
fun ref stdout(
  process: ProcessMonitor ref,
  data: Array[U8 val] iso)
: None val
```
#### Parameters

*   process: [ProcessMonitor](process-ProcessMonitor) ref
*   data: [Array](builtin-Array)\[[U8](builtin-U8) val\] iso

#### Returns

* [None](builtin-None) val

---

### stderr

ProcessMonitor calls this when new data is received on STDERR of the
forked process


```pony
fun ref stderr(
  process: ProcessMonitor ref,
  data: Array[U8 val] iso)
: None val
```
#### Parameters

*   process: [ProcessMonitor](process-ProcessMonitor) ref
*   data: [Array](builtin-Array)\[[U8](builtin-U8) val\] iso

#### Returns

* [None](builtin-None) val

---

### failed

ProcessMonitor calls this if we run into errors communicating with the
forked process.


```pony
fun ref failed(
  process: ProcessMonitor ref,
  err: (ExecveError val | ForkError val | KillError val | 
    PipeError val | Unsupported val | WaitpidError val | 
    WriteError val | CapError val))
: None val
```
#### Parameters

*   process: [ProcessMonitor](process-ProcessMonitor) ref
*   err: ([ExecveError](process-ExecveError) val | [ForkError](process-ForkError) val | [KillError](process-KillError) val | 
    [PipeError](process-PipeError) val | [Unsupported](process-Unsupported) val | [WaitpidError](process-WaitpidError) val | 
    [WriteError](process-WriteError) val | [CapError](process-CapError) val)

#### Returns

* [None](builtin-None) val

---

### expect

Called when the process monitor has been told to expect a certain quantity
of bytes. This allows nested notifiers to change the expected quantity,
which allows a lower level protocol to handle any framing.


```pony
fun ref expect(
  process: ProcessMonitor ref,
  qty: USize val)
: USize val
```
#### Parameters

*   process: [ProcessMonitor](process-ProcessMonitor) ref
*   qty: [USize](builtin-USize) val

#### Returns

* [USize](builtin-USize) val

---

### dispose

Call when ProcessMonitor terminates to cleanup ProcessNotify.
We return the exit code of the child process.


```pony
fun ref dispose(
  process: ProcessMonitor ref,
  child_exit_code: I32 val)
: None val
```
#### Parameters

*   process: [ProcessMonitor](process-ProcessMonitor) ref
*   child_exit_code: [I32](builtin-I32) val

#### Returns

* [None](builtin-None) val

---

