# ANSITerm

Handles ANSI escape codes from stdin.


```pony
actor tag ANSITerm
```

## Constructors

### create

```pony
new tag create(
  notify: ANSINotify iso,
  source: DisposableActor tag,
  timers: Timers tag = seq)
: ANSITerm tag^
```
#### Parameters

*   notify: [ANSINotify](term-ANSINotify) iso
*   source: [DisposableActor](builtin-DisposableActor) tag
*   timers: [Timers](time-Timers) tag = seq

#### Returns

* [ANSITerm](term-ANSITerm) tag^

---

## Public Behaviours

### apply

Receives input from stdin.


```pony
be apply(
  data: Array[U8 val] iso)
```
#### Parameters

*   data: [Array](builtin-Array)\[[U8](builtin-U8) val\] iso

---

### prompt

Pass a prompt along to the notifier.


```pony
be prompt(
  value: String val)
```
#### Parameters

*   value: [String](builtin-String) val

---

### size

```pony
be size()
```

---

### dispose

Stop accepting input, inform the notifier we have closed, and dispose of
our source.


```pony
be dispose()
```

---

## Private Behaviours

### _timeout

Our timer since receiving an ESC has expired. Send the buffered data as if
it was not an escape sequence.


```pony
be _timeout()
```

---

## Private Functions

### _size

Pass the window size to the notifier.


```pony
fun ref _size()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _mod

Set the modifier bools.


```pony
fun ref _mod()
: (Bool val , Bool val , Bool val)
```

#### Returns

* ([Bool](builtin-Bool) val , [Bool](builtin-Bool) val , [Bool](builtin-Bool) val)

---

### _keypad

An extended key.


```pony
fun ref _keypad()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _up

Up arrow.


```pony
fun ref _up()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _down

Down arrow.


```pony
fun ref _down()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _left

Left arrow.


```pony
fun ref _left()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _right

Right arrow.


```pony
fun ref _right()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _delete

Delete key.


```pony
fun ref _delete()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _insert

Insert key.


```pony
fun ref _insert()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _home

Home key.


```pony
fun ref _home()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _end

End key.


```pony
fun ref _end()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _page_up

Page up key.


```pony
fun ref _page_up()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _page_down

Page down key.


```pony
fun ref _page_down()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _fn_key

Function key.


```pony
fun ref _fn_key(
  i: U8 val)
: None val
```
#### Parameters

*   i: [U8](builtin-U8) val

#### Returns

* [None](builtin-None) val

---

### _esc_flush

Pass a partial or unrecognised escape sequence to the notifier.


```pony
fun ref _esc_flush()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _esc_clear

Clear the escape state.


```pony
fun ref _esc_clear()
: None val
```

#### Returns

* [None](builtin-None) val

---

